@@ -13,123 +13,138 @@
 #define RECURSOR_TRAPS_OID RECURSOR_OID, 10, 0
 #define RECURSOR_TRAP_OBJECTS_OID RECURSOR_OID, 11
 
-static const oid trapReasonOID[] = { RECURSOR_TRAP_OBJECTS_OID, 1, 0 };
-static const oid customTrapOID[] = { RECURSOR_TRAPS_OID, 1 };
+static const oid trapReasonOID[] = {RECURSOR_TRAP_OBJECTS_OID, 1, 0};
+static const oid customTrapOID[] = {RECURSOR_TRAPS_OID, 1};
 
-static const oid questionsOID[] = { RECURSOR_STATS_OID, 1 };
-static const oid ipv6QuestionsOID[] = { RECURSOR_STATS_OID, 2 };
-static const oid tcpQuestionsOID[] = { RECURSOR_STATS_OID, 3 };
-static const oid cacheHitsOID[] = { RECURSOR_STATS_OID, 4 };
-static const oid cacheMissesOID[] = { RECURSOR_STATS_OID, 5 };
-static const oid cacheEntriesOID[] = { RECURSOR_STATS_OID, 6 };
-static const oid cacheBytesOID[] = { RECURSOR_STATS_OID, 7 };
-static const oid packetcacheHitsOID[] = { RECURSOR_STATS_OID, 8 };
-static const oid packetcacheMissesOID[] = { RECURSOR_STATS_OID, 9 };
-static const oid packetcacheEntriesOID[] = { RECURSOR_STATS_OID, 10 };
-static const oid packetcacheBytesOID[] = { RECURSOR_STATS_OID, 11 };
-static const oid mallocBytesOID[] = { RECURSOR_STATS_OID, 12 };
-static const oid servfailAnswersOID[] = { RECURSOR_STATS_OID, 13 };
-static const oid nxdomainAnswersOID[] = { RECURSOR_STATS_OID, 14 };
-static const oid noerrorAnswersOID[] = { RECURSOR_STATS_OID, 15 };
-static const oid unauthorizedUdpOID[] = { RECURSOR_STATS_OID, 16 };
-static const oid unauthorizedTcpOID[] = { RECURSOR_STATS_OID, 17 };
-static const oid tcpClientOverflowOID[] = { RECURSOR_STATS_OID, 18 };
-static const oid clientParseErrorsOID[] = { RECURSOR_STATS_OID, 19 };
-static const oid serverParseErrorsOID[] = { RECURSOR_STATS_OID, 20 };
-static const oid tooOldDropsOID[] = { RECURSOR_STATS_OID, 21 };
-static const oid answers01OID[] = { RECURSOR_STATS_OID, 22 };
-static const oid answers110OID[] = { RECURSOR_STATS_OID, 23 };
-static const oid answers10100OID[] = { RECURSOR_STATS_OID, 24 };
-static const oid answers1001000OID[] = { RECURSOR_STATS_OID, 25 };
-static const oid answersSlowOID[] = { RECURSOR_STATS_OID, 26 };
-static const oid auth4Answers01OID[] = { RECURSOR_STATS_OID, 27 };
-static const oid auth4Answers110OID[] = { RECURSOR_STATS_OID, 28 };
-static const oid auth4Answers10100OID[] = { RECURSOR_STATS_OID, 29 };
-static const oid auth4Answers1001000OID[] = { RECURSOR_STATS_OID, 30 };
-static const oid auth4AnswersslowOID[] = { RECURSOR_STATS_OID, 31 };
-static const oid auth6Answers01OID[] = { RECURSOR_STATS_OID, 32 };
-static const oid auth6Answers110OID[] = { RECURSOR_STATS_OID, 33 };
-static const oid auth6Answers10100OID[] = { RECURSOR_STATS_OID, 34 };
-static const oid auth6Answers1001000OID[] = { RECURSOR_STATS_OID, 35 };
-static const oid auth6AnswersSlowOID[] = { RECURSOR_STATS_OID, 36 };
-static const oid qaLatencyOID[] = { RECURSOR_STATS_OID, 37 };
-static const oid unexpectedPacketsOID[] = { RECURSOR_STATS_OID, 38 };
-static const oid caseMismatchesOID[] = { RECURSOR_STATS_OID, 39 };
-static const oid spoofPreventsOID[] = { RECURSOR_STATS_OID, 40 };
-static const oid nssetInvalidationsOID[] = { RECURSOR_STATS_OID, 41 };
-static const oid resourceLimitsOID[] = { RECURSOR_STATS_OID, 42 };
-static const oid overCapacityDropsOID[] = { RECURSOR_STATS_OID, 43 };
-static const oid policyDropsOID[] = { RECURSOR_STATS_OID, 44 };
-static const oid noPacketErrorOID[] = { RECURSOR_STATS_OID, 45 };
-static const oid dlgOnlyDropsOID[] = { RECURSOR_STATS_OID, 46 };
-static const oid ignoredPacketsOID[] = { RECURSOR_STATS_OID, 47 };
-static const oid maxMthreadStackOID[] = { RECURSOR_STATS_OID, 48 };
-static const oid negcacheEntriesOID[] = { RECURSOR_STATS_OID, 49 };
-static const oid throttleEntriesOID[] = { RECURSOR_STATS_OID, 50 };
-static const oid nsspeedsEntriesOID[] = { RECURSOR_STATS_OID, 51 };
-static const oid failedHostEntriesOID[] = { RECURSOR_STATS_OID, 52 };
-static const oid concurrentQueriesOID[] = { RECURSOR_STATS_OID, 53 };
-static const oid securityStatusOID[] = { RECURSOR_STATS_OID, 54 };
-static const oid outgoingTimeoutsOID[] = { RECURSOR_STATS_OID, 55 };
-static const oid outgoing4TimeoutsOID[] = { RECURSOR_STATS_OID, 56 };
-static const oid outgoing6TimeoutsOID[] = { RECURSOR_STATS_OID, 57 };
-static const oid tcpOutqueriesOID[] = { RECURSOR_STATS_OID, 58 };
-static const oid allOutqueriesOID[] = { RECURSOR_STATS_OID, 59 };
-static const oid ipv6OutqueriesOID[] = { RECURSOR_STATS_OID, 60 };
-static const oid throttledOutqueriesOID[] = { RECURSOR_STATS_OID, 61 };
-static const oid dontOutqueriesOID[] = { RECURSOR_STATS_OID, 62 };
-static const oid unreachablesOID[] = { RECURSOR_STATS_OID, 63 };
-static const oid chainResendsOID[] = { RECURSOR_STATS_OID, 64 };
-static const oid tcpClientsOID[] = { RECURSOR_STATS_OID, 65 };
+static const oid questionsOID[] = {RECURSOR_STATS_OID, 1};
+static const oid ipv6QuestionsOID[] = {RECURSOR_STATS_OID, 2};
+static const oid tcpQuestionsOID[] = {RECURSOR_STATS_OID, 3};
+static const oid cacheHitsOID[] = {RECURSOR_STATS_OID, 4};
+static const oid cacheMissesOID[] = {RECURSOR_STATS_OID, 5};
+static const oid cacheEntriesOID[] = {RECURSOR_STATS_OID, 6};
+static const oid cacheBytesOID[] = {RECURSOR_STATS_OID, 7};
+static const oid packetcacheHitsOID[] = {RECURSOR_STATS_OID, 8};
+static const oid packetcacheMissesOID[] = {RECURSOR_STATS_OID, 9};
+static const oid packetcacheEntriesOID[] = {RECURSOR_STATS_OID, 10};
+static const oid packetcacheBytesOID[] = {RECURSOR_STATS_OID, 11};
+static const oid mallocBytesOID[] = {RECURSOR_STATS_OID, 12};
+static const oid servfailAnswersOID[] = {RECURSOR_STATS_OID, 13};
+static const oid nxdomainAnswersOID[] = {RECURSOR_STATS_OID, 14};
+static const oid noerrorAnswersOID[] = {RECURSOR_STATS_OID, 15};
+static const oid unauthorizedUdpOID[] = {RECURSOR_STATS_OID, 16};
+static const oid unauthorizedTcpOID[] = {RECURSOR_STATS_OID, 17};
+static const oid tcpClientOverflowOID[] = {RECURSOR_STATS_OID, 18};
+static const oid clientParseErrorsOID[] = {RECURSOR_STATS_OID, 19};
+static const oid serverParseErrorsOID[] = {RECURSOR_STATS_OID, 20};
+static const oid tooOldDropsOID[] = {RECURSOR_STATS_OID, 21};
+static const oid answers01OID[] = {RECURSOR_STATS_OID, 22};
+static const oid answers110OID[] = {RECURSOR_STATS_OID, 23};
+static const oid answers10100OID[] = {RECURSOR_STATS_OID, 24};
+static const oid answers1001000OID[] = {RECURSOR_STATS_OID, 25};
+static const oid answersSlowOID[] = {RECURSOR_STATS_OID, 26};
+static const oid auth4Answers01OID[] = {RECURSOR_STATS_OID, 27};
+static const oid auth4Answers110OID[] = {RECURSOR_STATS_OID, 28};
+static const oid auth4Answers10100OID[] = {RECURSOR_STATS_OID, 29};
+static const oid auth4Answers1001000OID[] = {RECURSOR_STATS_OID, 30};
+static const oid auth4AnswersslowOID[] = {RECURSOR_STATS_OID, 31};
+static const oid auth6Answers01OID[] = {RECURSOR_STATS_OID, 32};
+static const oid auth6Answers110OID[] = {RECURSOR_STATS_OID, 33};
+static const oid auth6Answers10100OID[] = {RECURSOR_STATS_OID, 34};
+static const oid auth6Answers1001000OID[] = {RECURSOR_STATS_OID, 35};
+static const oid auth6AnswersSlowOID[] = {RECURSOR_STATS_OID, 36};
+static const oid qaLatencyOID[] = {RECURSOR_STATS_OID, 37};
+static const oid unexpectedPacketsOID[] = {RECURSOR_STATS_OID, 38};
+static const oid caseMismatchesOID[] = {RECURSOR_STATS_OID, 39};
+static const oid spoofPreventsOID[] = {RECURSOR_STATS_OID, 40};
+static const oid nssetInvalidationsOID[] = {RECURSOR_STATS_OID, 41};
+static const oid resourceLimitsOID[] = {RECURSOR_STATS_OID, 42};
+static const oid overCapacityDropsOID[] = {RECURSOR_STATS_OID, 43};
+static const oid policyDropsOID[] = {RECURSOR_STATS_OID, 44};
+static const oid noPacketErrorOID[] = {RECURSOR_STATS_OID, 45};
+static const oid dlgOnlyDropsOID[] = {RECURSOR_STATS_OID, 46};
+static const oid ignoredPacketsOID[] = {RECURSOR_STATS_OID, 47};
+static const oid maxMthreadStackOID[] = {RECURSOR_STATS_OID, 48};
+static const oid negcacheEntriesOID[] = {RECURSOR_STATS_OID, 49};
+static const oid throttleEntriesOID[] = {RECURSOR_STATS_OID, 50};
+static const oid nsspeedsEntriesOID[] = {RECURSOR_STATS_OID, 51};
+static const oid failedHostEntriesOID[] = {RECURSOR_STATS_OID, 52};
+static const oid concurrentQueriesOID[] = {RECURSOR_STATS_OID, 53};
+static const oid securityStatusOID[] = {RECURSOR_STATS_OID, 54};
+static const oid outgoingTimeoutsOID[] = {RECURSOR_STATS_OID, 55};
+static const oid outgoing4TimeoutsOID[] = {RECURSOR_STATS_OID, 56};
+static const oid outgoing6TimeoutsOID[] = {RECURSOR_STATS_OID, 57};
+static const oid tcpOutqueriesOID[] = {RECURSOR_STATS_OID, 58};
+static const oid allOutqueriesOID[] = {RECURSOR_STATS_OID, 59};
+static const oid ipv6OutqueriesOID[] = {RECURSOR_STATS_OID, 60};
+static const oid throttledOutqueriesOID[] = {RECURSOR_STATS_OID, 61};
+static const oid dontOutqueriesOID[] = {RECURSOR_STATS_OID, 62};
+static const oid unreachablesOID[] = {RECURSOR_STATS_OID, 63};
+static const oid chainResendsOID[] = {RECURSOR_STATS_OID, 64};
+static const oid tcpClientsOID[] = {RECURSOR_STATS_OID, 65};
 #ifdef __linux__
-static const oid udpRecvbufErrorsOID[] = { RECURSOR_STATS_OID, 66 };
-static const oid udpSndbufErrorsOID[] = { RECURSOR_STATS_OID, 67 };
-static const oid udpNoportErrorsOID[] = { RECURSOR_STATS_OID, 68 };
-static const oid udpinErrorsOID[] = { RECURSOR_STATS_OID, 69 };
+static const oid udpRecvbufErrorsOID[] = {RECURSOR_STATS_OID, 66};
+static const oid udpSndbufErrorsOID[] = {RECURSOR_STATS_OID, 67};
+static const oid udpNoportErrorsOID[] = {RECURSOR_STATS_OID, 68};
+static const oid udpinErrorsOID[] = {RECURSOR_STATS_OID, 69};
 #endif /* __linux__ */
-static const oid ednsPingMatchesOID[] = { RECURSOR_STATS_OID, 70 };
-static const oid ednsPingMismatchesOID[] = { RECURSOR_STATS_OID, 71 };
-static const oid dnssecQueriesOID[] = { RECURSOR_STATS_OID, 72 };
-static const oid nopingOutqueriesOID[] = { RECURSOR_STATS_OID, 73 };
-static const oid noednsOutqueriesOID[] = { RECURSOR_STATS_OID, 74 };
-static const oid uptimeOID[] = { RECURSOR_STATS_OID, 75 };
-static const oid realMemoryUsageOID[] = { RECURSOR_STATS_OID, 76 };
-static const oid fdUsageOID[] = { RECURSOR_STATS_OID, 77 };
-static const oid userMsecOID[] = { RECURSOR_STATS_OID, 78 };
-static const oid sysMsecOID[] = { RECURSOR_STATS_OID, 79 };
-static const oid dnssecValidationsOID[] = { RECURSOR_STATS_OID, 80 };
-static const oid dnssecResultInsecureOID[] = { RECURSOR_STATS_OID, 81 };
-static const oid dnssecResultSecureOID[] = { RECURSOR_STATS_OID, 82 };
-static const oid dnssecResultBogusOID[] = { RECURSOR_STATS_OID, 83 };
-static const oid dnssecResultIndeterminateOID[] = { RECURSOR_STATS_OID, 84 };
-static const oid dnssecResultNtaOID[] = { RECURSOR_STATS_OID, 85 };
-static const oid policyResultNoactionOID[] = { RECURSOR_STATS_OID, 86 };
-static const oid policyResultDropOID[] = { RECURSOR_STATS_OID, 87 };
-static const oid policyResultNxdomainOID[] = { RECURSOR_STATS_OID, 88 };
-static const oid policyResultNodataOID[] = { RECURSOR_STATS_OID, 89 };
-static const oid policyResultTruncateOID[] = { RECURSOR_STATS_OID, 90 };
-static const oid policyResultCustomOID[] = { RECURSOR_STATS_OID, 91 };
-static const oid queryPipeFullDropsOID[] = { RECURSOR_STATS_OID, 92 };
-static const oid truncatedDropsOID[] = { RECURSOR_STATS_OID, 93 };
-static const oid emptyQueriesOID[] = { RECURSOR_STATS_OID, 94 };
-static const oid dnssecAuthenticDataQueriesOID[] = { RECURSOR_STATS_OID, 95 };
-static const oid dnssecCheckDisabledQueriesOID[] = { RECURSOR_STATS_OID, 96 };
-static const oid variableResponsesOID[] = { RECURSOR_STATS_OID, 97 };
-static const oid specialMemoryUsageOID[] = { RECURSOR_STATS_OID, 98 };
-static const oid rebalancedQueriesOID[] = { RECURSOR_STATS_OID, 99 };
-static const oid qnameMinFallbackSuccessOID[] = { RECURSOR_STATS_OID, 100 };
-static const oid proxyProtocolInvalidOID[] = { RECURSOR_STATS_OID, 101 };
-static const oid recordCacheContendedOID[] = { RECURSOR_STATS_OID, 102 };
-static const oid recordCacheAcquiredOID[] = { RECURSOR_STATS_OID, 103 };
-static const oid nodLookupsDroppedOversizeOID[] = { RECURSOR_STATS_OID, 104 };
-static const oid taskQueuePushedOID[] = { RECURSOR_STATS_OID, 105 };
-static const oid taskQueueExpiredOID[] = { RECURSOR_STATS_OID, 106 };
-static const oid taskQueueSizeOID[] = { RECURSOR_STATS_OID, 107 };
-static const oid aggressiveNSECCacheEntriesOID[] = { RECURSOR_STATS_OID, 108 };
-static const oid aggressiveNSECCacheNSECHitsOID[] = { RECURSOR_STATS_OID, 109 };
-static const oid aggressiveNSECCacheNSEC3HitsOID[] = { RECURSOR_STATS_OID, 110 };
-static const oid aggressiveNSECCacheNSECWCHitsOID[] = { RECURSOR_STATS_OID, 111 };
-static const oid aggressiveNSECCacheNSEC3WCHitsOID[] = { RECURSOR_STATS_OID, 112 };
+static const oid ednsPingMatchesOID[] = {RECURSOR_STATS_OID, 70};
+static const oid ednsPingMismatchesOID[] = {RECURSOR_STATS_OID, 71};
+static const oid dnssecQueriesOID[] = {RECURSOR_STATS_OID, 72};
+static const oid nopingOutqueriesOID[] = {RECURSOR_STATS_OID, 73};
+static const oid noednsOutqueriesOID[] = {RECURSOR_STATS_OID, 74};
+static const oid uptimeOID[] = {RECURSOR_STATS_OID, 75};
+static const oid realMemoryUsageOID[] = {RECURSOR_STATS_OID, 76};
+static const oid fdUsageOID[] = {RECURSOR_STATS_OID, 77};
+static const oid userMsecOID[] = {RECURSOR_STATS_OID, 78};
+static const oid sysMsecOID[] = {RECURSOR_STATS_OID, 79};
+static const oid dnssecValidationsOID[] = {RECURSOR_STATS_OID, 80};
+static const oid dnssecResultInsecureOID[] = {RECURSOR_STATS_OID, 81};
+static const oid dnssecResultSecureOID[] = {RECURSOR_STATS_OID, 82};
+static const oid dnssecResultBogusOID[] = {RECURSOR_STATS_OID, 83};
+static const oid dnssecResultIndeterminateOID[] = {RECURSOR_STATS_OID, 84};
+static const oid dnssecResultNtaOID[] = {RECURSOR_STATS_OID, 85};
+static const oid policyResultNoactionOID[] = {RECURSOR_STATS_OID, 86};
+static const oid policyResultDropOID[] = {RECURSOR_STATS_OID, 87};
+static const oid policyResultNxdomainOID[] = {RECURSOR_STATS_OID, 88};
+static const oid policyResultNodataOID[] = {RECURSOR_STATS_OID, 89};
+static const oid policyResultTruncateOID[] = {RECURSOR_STATS_OID, 90};
+static const oid policyResultCustomOID[] = {RECURSOR_STATS_OID, 91};
+static const oid queryPipeFullDropsOID[] = {RECURSOR_STATS_OID, 92};
+static const oid truncatedDropsOID[] = {RECURSOR_STATS_OID, 93};
+static const oid emptyQueriesOID[] = {RECURSOR_STATS_OID, 94};
+static const oid dnssecAuthenticDataQueriesOID[] = {RECURSOR_STATS_OID, 95};
+static const oid dnssecCheckDisabledQueriesOID[] = {RECURSOR_STATS_OID, 96};
+static const oid variableResponsesOID[] = {RECURSOR_STATS_OID, 97};
+static const oid specialMemoryUsageOID[] = {RECURSOR_STATS_OID, 98};
+static const oid rebalancedQueriesOID[] = {RECURSOR_STATS_OID, 99};
+static const oid qnameMinFallbackSuccessOID[] = {RECURSOR_STATS_OID, 100};
+static const oid proxyProtocolInvalidOID[] = {RECURSOR_STATS_OID, 101};
+static const oid recordCacheContendedOID[] = {RECURSOR_STATS_OID, 102};
+static const oid recordCacheAcquiredOID[] = {RECURSOR_STATS_OID, 103};
+static const oid nodLookupsDroppedOversizeOID[] = {RECURSOR_STATS_OID, 104};
+static const oid taskQueuePushedOID[] = {RECURSOR_STATS_OID, 105};
+static const oid taskQueueExpiredOID[] = {RECURSOR_STATS_OID, 106};
+static const oid taskQueueSizeOID[] = {RECURSOR_STATS_OID, 107};
+static const oid aggressiveNSECCacheEntriesOID[] = {RECURSOR_STATS_OID, 108};
+static const oid aggressiveNSECCacheNSECHitsOID[] = {RECURSOR_STATS_OID, 109};
+static const oid aggressiveNSECCacheNSEC3HitsOID[] = {RECURSOR_STATS_OID, 110};
+static const oid aggressiveNSECCacheNSECWCHitsOID[] = {RECURSOR_STATS_OID, 111};
+static const oid aggressiveNSECCacheNSEC3WCHitsOID[] = {RECURSOR_STATS_OID, 112};
+static const oid dotOutqueriesOID[] = {RECURSOR_STATS_OID, 113};
+static const oid dns64PrefixAnswers[] = {RECURSOR_STATS_OID, 114};
+static const oid almostExpiredPushed[] = {RECURSOR_STATS_OID, 115};
+static const oid almostExpiredRun[] = {RECURSOR_STATS_OID, 116};
+static const oid almostExpiredExceptions[] = {RECURSOR_STATS_OID, 117};
+#ifdef __linux__
+static const oid udpInCsumErrorsOID[] = {RECURSOR_STATS_OID, 118};
+static const oid udp6RecvbufErrorsOID[] = {RECURSOR_STATS_OID, 119};
+static const oid udp6SndbufErrorsOID[] = {RECURSOR_STATS_OID, 120};
+static const oid udp6NoportErrorsOID[] = {RECURSOR_STATS_OID, 121};
+static const oid udp6InErrorsOID[] = {RECURSOR_STATS_OID, 122};
+static const oid udp6InCsumErrorsOID[] = {RECURSOR_STATS_OID, 123};
+#endif /* __linux__ */
+static const oid sourceDisallowedNotifyOID[] = {RECURSOR_STATS_OID, 124};
+static const oid zoneDisallowedNotifyOID[] = {RECURSOR_STATS_OID, 125};
 
 static std::unordered_map<oid, std::string> s_statsMap;
 
@@ -159,7 +174,8 @@ static int handleCounter64Stats(netsnmp_mib_handler* handler,
   boost::optional<uint64_t> value = getStatByName(it->second);
   if (value) {
     return RecursorSNMPAgent::setCounter64Value(requests, *value);
-  } else {
+  }
+  else {
     return RecursorSNMPAgent::setCounter64Value(requests, 0);
   }
 }
@@ -183,19 +199,18 @@ static int handleDisabledCounter64Stats(netsnmp_mib_handler* handler,
 static void registerCounter64Stat(const std::string& name, const oid statOID[], size_t statOIDLength)
 {
   if (statOIDLength != OID_LENGTH(questionsOID)) {
-    g_log<<Logger::Error<<"Invalid OID for SNMP Counter64 statistic "<<name<<endl;
+    g_log << Logger::Error << "Invalid OID for SNMP Counter64 statistic " << name << endl;
     return;
   }
 
   if (s_statsMap.find(statOID[statOIDLength - 1]) != s_statsMap.end()) {
-    g_log<<Logger::Error<<"OID for SNMP Counter64 statistic "<<name<<" has already been registered"<<endl;
+    g_log << Logger::Error << "OID for SNMP Counter64 statistic " << name << " has already been registered" << endl;
     return;
   }
 
   s_statsMap[statOID[statOIDLength - 1]] = name.c_str();
   netsnmp_register_scalar(netsnmp_create_handler_registration(name.c_str(),
-                                                              isStatBlacklisted(StatComponent::SNMP, name) ?
-                                                              handleDisabledCounter64Stats : handleCounter64Stats,
+                                                              isStatDisabled(StatComponent::SNMP, name) ? handleDisabledCounter64Stats : handleCounter64Stats,
                                                               statOID,
                                                               statOIDLength,
                                                               HANDLER_CAN_RONLY));
@@ -215,7 +230,7 @@ bool RecursorSNMPAgent::sendCustomTrap(const std::string& reason)
                             snmpTrapOIDLen,
                             ASN_OBJECT_ID,
                             customTrapOID,
-                            OID_LENGTH(customTrapOID)  * sizeof(oid));
+                            OID_LENGTH(customTrapOID) * sizeof(oid));
 
   snmp_varlist_add_variable(&varList,
                             trapReasonOID,
@@ -229,8 +244,8 @@ bool RecursorSNMPAgent::sendCustomTrap(const std::string& reason)
   return true;
 }
 
-
-RecursorSNMPAgent::RecursorSNMPAgent(const std::string& name, const std::string& masterSocket): SNMPAgent(name, masterSocket)
+RecursorSNMPAgent::RecursorSNMPAgent(const std::string& name, const std::string& masterSocket) :
+  SNMPAgent(name, masterSocket)
 {
 #ifdef HAVE_NET_SNMP
   /* This is done so that the statistics maps are
@@ -254,6 +269,8 @@ RecursorSNMPAgent::RecursorSNMPAgent(const std::string& name, const std::string&
   registerCounter64Stat("noerror-answers", noerrorAnswersOID, OID_LENGTH(noerrorAnswersOID));
   registerCounter64Stat("unauthorized-udp", unauthorizedUdpOID, OID_LENGTH(unauthorizedUdpOID));
   registerCounter64Stat("unauthorized-tcp", unauthorizedTcpOID, OID_LENGTH(unauthorizedTcpOID));
+  registerCounter64Stat("source-disallowed-notify", sourceDisallowedNotifyOID, OID_LENGTH(sourceDisallowedNotifyOID));
+  registerCounter64Stat("zone-disallowed-notify", zoneDisallowedNotifyOID, OID_LENGTH(zoneDisallowedNotifyOID));
   registerCounter64Stat("tcp-client-overflow", tcpClientOverflowOID, OID_LENGTH(tcpClientOverflowOID));
   registerCounter64Stat("client-parse-errors", clientParseErrorsOID, OID_LENGTH(clientParseErrorsOID));
   registerCounter64Stat("server-parse-errors", serverParseErrorsOID, OID_LENGTH(serverParseErrorsOID));
@@ -312,6 +329,12 @@ RecursorSNMPAgent::RecursorSNMPAgent(const std::string& name, const std::string&
   registerCounter64Stat("udp-sndbuf-errors", udpSndbufErrorsOID, OID_LENGTH(udpSndbufErrorsOID));
   registerCounter64Stat("udp-noport-errors", udpNoportErrorsOID, OID_LENGTH(udpNoportErrorsOID));
   registerCounter64Stat("udp-in-errors", udpinErrorsOID, OID_LENGTH(udpinErrorsOID));
+  registerCounter64Stat("udp-in-csums-errors", udpInCsumErrorsOID, OID_LENGTH(udpInCsumErrorsOID));
+  registerCounter64Stat("udp6-recvbuf-errors", udp6RecvbufErrorsOID, OID_LENGTH(udp6RecvbufErrorsOID));
+  registerCounter64Stat("udp6-sndbuf-errors", udp6SndbufErrorsOID, OID_LENGTH(udp6SndbufErrorsOID));
+  registerCounter64Stat("udp6-noport-errors", udp6NoportErrorsOID, OID_LENGTH(udp6NoportErrorsOID));
+  registerCounter64Stat("udp6-in-errors", udp6InErrorsOID, OID_LENGTH(udp6InErrorsOID));
+  registerCounter64Stat("udp6-in-csums-errors", udp6InCsumErrorsOID, OID_LENGTH(udp6InCsumErrorsOID));
 #endif /* __linux__ */
   registerCounter64Stat("edns-ping-matches", ednsPingMatchesOID, OID_LENGTH(ednsPingMatchesOID));
   registerCounter64Stat("edns-ping-mismatches", ednsPingMismatchesOID, OID_LENGTH(ednsPingMismatchesOID));
@@ -351,5 +374,10 @@ RecursorSNMPAgent::RecursorSNMPAgent(const std::string& name, const std::string&
   registerCounter64Stat("aggressive-nsec-cache-nsec3-hits", aggressiveNSECCacheNSEC3HitsOID, OID_LENGTH(aggressiveNSECCacheNSEC3HitsOID));
   registerCounter64Stat("aggressive-nsec-cache-nsec-wc-hits", aggressiveNSECCacheNSECWCHitsOID, OID_LENGTH(aggressiveNSECCacheNSECWCHitsOID));
   registerCounter64Stat("aggressive-nsec-cache-nsec-wc3-hits", aggressiveNSECCacheNSEC3WCHitsOID, OID_LENGTH(aggressiveNSECCacheNSEC3WCHitsOID));
+  registerCounter64Stat("dot-outqueries", dotOutqueriesOID, OID_LENGTH(dotOutqueriesOID));
+  registerCounter64Stat("dns64-prefix-answers", dns64PrefixAnswers, OID_LENGTH(dns64PrefixAnswers));
+  registerCounter64Stat("almost-expired-pushed", almostExpiredPushed, OID_LENGTH(almostExpiredPushed));
+  registerCounter64Stat("almost-expired-run", almostExpiredRun, OID_LENGTH(almostExpiredRun));
+  registerCounter64Stat("almost-expired-exceptions", almostExpiredExceptions, OID_LENGTH(almostExpiredExceptions));
 #endif /* HAVE_NET_SNMP */
 }
