@@ -31,7 +31,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_denial_nowrap)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a.example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a.example.org."), QType::NSEC)] = pair;
 
   /* add wildcard denial */
   recordContents.clear();
@@ -44,7 +44,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_denial_nowrap)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(DNSName("example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("example.org."), QType::NSEC)] = pair;
 
   dState denialState = getDenial(denialMap, DNSName("b.example.org."), QType::A, false, false);
   BOOST_CHECK_EQUAL(denialState, dState::NXDOMAIN);
@@ -80,7 +80,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_denial_wrap_case_1)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("z.example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("z.example.org."), QType::NSEC)] = pair;
 
   dState denialState = getDenial(denialMap, DNSName("a.example.org."), QType::A, false, false);
   BOOST_CHECK_EQUAL(denialState, dState::NXDOMAIN);
@@ -116,7 +116,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_denial_wrap_case_2)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("y.example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("y.example.org."), QType::NSEC)] = pair;
 
   dState denialState = getDenial(denialMap, DNSName("z.example.org."), QType::A, false, false);
   BOOST_CHECK_EQUAL(denialState, dState::NXDOMAIN);
@@ -152,7 +152,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_denial_only_one_nsec)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a.example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a.example.org."), QType::NSEC)] = pair;
 
   dState denialState = getDenial(denialMap, DNSName("b.example.org."), QType::A, false, false);
   BOOST_CHECK_EQUAL(denialState, dState::NXDOMAIN);
@@ -188,7 +188,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_root_nxd_denial)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a."), QType::NSEC)] = pair;
 
   /* add wildcard denial */
   recordContents.clear();
@@ -201,7 +201,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_root_nxd_denial)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(DNSName("."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("."), QType::NSEC)] = pair;
 
   dState denialState = getDenial(denialMap, DNSName("b."), QType::A, false, false);
   BOOST_CHECK_EQUAL(denialState, dState::NXDOMAIN);
@@ -223,7 +223,8 @@ BOOST_AUTO_TEST_CASE(test_nsec_ancestor_nxqtype_denial)
     The RRSIG from "." denies the existence of any type except NS at a.
     However since it's an ancestor delegation NSEC (NS bit set, SOA bit clear,
     signer field that is shorter than the owner name of the NSEC RR) it can't
-    be used to deny anything except the whole name or a DS.
+    be used to deny anything except the whole name (which does not make sense here)
+    or a DS.
   */
   addNSECRecordToLW(DNSName("a."), DNSName("b."), {QType::NS}, 600, records);
   recordContents.insert(records.at(0).d_content);
@@ -235,7 +236,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_ancestor_nxqtype_denial)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a."), QType::NSEC)] = pair;
 
   /* RFC 6840 section 4.1 "Clarifications on Nonexistence Proofs":
      Ancestor delegation NSEC or NSEC3 RRs MUST NOT be used to assume
@@ -244,7 +245,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_ancestor_nxqtype_denial)
      owner name regardless of type.
   */
 
-  dState denialState = getDenial(denialMap, DNSName("a."), QType::A, false, false);
+  dState denialState = getDenial(denialMap, DNSName("a."), QType::A, false, true);
   /* no data means the qname/qtype is not denied, because an ancestor
      delegation NSEC can only deny the DS */
   BOOST_CHECK_EQUAL(denialState, dState::NODENIAL);
@@ -257,6 +258,38 @@ BOOST_AUTO_TEST_CASE(test_nsec_ancestor_nxqtype_denial)
   BOOST_CHECK_EQUAL(denialState, dState::NXQTYPE);
 }
 
+BOOST_AUTO_TEST_CASE(test_nsec_ds_denial_from_child)
+{
+  initSR();
+
+  testkeysset_t keys;
+  generateKeyMaterial(DNSName("org."), DNSSECKeeper::ECDSA256, DNSSECKeeper::DIGEST_SHA256, keys);
+  generateKeyMaterial(DNSName("example.org."), DNSSECKeeper::ECDSA256, DNSSECKeeper::DIGEST_SHA256, keys);
+
+  vector<DNSRecord> records;
+
+  sortedRecords_t recordContents;
+  vector<shared_ptr<RRSIGRecordContent>> signatureContents;
+
+  addNSECRecordToLW(DNSName("example.org."), DNSName("a.example.org"), {QType::A, QType::TXT, QType::RRSIG, QType::NSEC}, 600, records);
+  recordContents.insert(records.at(0).d_content);
+  addRRSIG(keys, records, DNSName("example.org."), 300);
+  signatureContents.push_back(getRR<RRSIGRecordContent>(records.at(1)));
+  records.clear();
+
+  ContentSigPair pair;
+  pair.records = recordContents;
+  pair.signatures = signatureContents;
+  cspmap_t denialMap;
+  denialMap[std::pair(DNSName("example.org."), QType::NSEC)] = pair;
+
+  /* check that this NSEC from the child zone can deny a AAAA at the apex */
+  BOOST_CHECK_EQUAL(getDenial(denialMap, DNSName("example.org."), QType::AAAA, false, true, true), dState::NXQTYPE);
+
+  /* but not that the DS does not exist, since we need the parent for that */
+  BOOST_CHECK_EQUAL(getDenial(denialMap, DNSName("example.org."), QType::DS, false, true, true), dState::NODENIAL);
+}
+
 BOOST_AUTO_TEST_CASE(test_nsec_insecure_delegation_denial)
 {
   initSR();
@@ -291,7 +324,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_insecure_delegation_denial)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a."), QType::NSEC)] = pair;
 
   /* Insecure because the NS is not set, so while it does
      denies the DS, it can't prove an insecure delegation */
@@ -321,13 +354,43 @@ BOOST_AUTO_TEST_CASE(test_nsec_nxqtype_cname)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a.powerdns.com."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a.powerdns.com."), QType::NSEC)] = pair;
 
   /* this NSEC is not valid to deny a.powerdns.com|A since it states that a CNAME exists */
   dState denialState = getDenial(denialMap, DNSName("a.powerdns.com."), QType::A, true, true);
   BOOST_CHECK_EQUAL(denialState, dState::NODENIAL);
 }
 
+BOOST_AUTO_TEST_CASE(test_nsec3_nxqtype_ds)
+{
+  initSR();
+
+  testkeysset_t keys;
+  generateKeyMaterial(DNSName("powerdns.com."), DNSSECKeeper::ECDSA256, DNSSECKeeper::DIGEST_SHA256, keys);
+
+  vector<DNSRecord> records;
+
+  sortedRecords_t recordContents;
+  vector<shared_ptr<RRSIGRecordContent>> signatureContents;
+
+  addNSEC3UnhashedRecordToLW(DNSName("powerdns.com."), DNSName("powerdns.com."), "whatever", {QType::A}, 600, records);
+  recordContents.insert(records.at(0).d_content);
+  addRRSIG(keys, records, DNSName("powerdns.com."), 300);
+  signatureContents.push_back(getRR<RRSIGRecordContent>(records.at(1)));
+
+  ContentSigPair pair;
+  pair.records = recordContents;
+  pair.signatures = signatureContents;
+  cspmap_t denialMap;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  records.clear();
+
+  /* this NSEC3 is not valid to deny the DS since it is from the child zone */
+  BOOST_CHECK_EQUAL(getDenial(denialMap, DNSName("powerdns.com."), QType::DS, false, true), dState::NODENIAL);
+  /* AAAA should be fine, though */
+  BOOST_CHECK_EQUAL(getDenial(denialMap, DNSName("powerdns.com."), QType::AAAA, false, true), dState::NXQTYPE);
+}
+
 BOOST_AUTO_TEST_CASE(test_nsec3_nxqtype_cname)
 {
   initSR();
@@ -349,7 +412,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_nxqtype_cname)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
   records.clear();
 
   /* this NSEC3 is not valid to deny a.powerdns.com|A since it states that a CNAME exists */
@@ -379,7 +442,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_nxdomain_denial_missing_wildcard)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a.powerdns.com."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a.powerdns.com."), QType::NSEC)] = pair;
 
   dState denialState = getDenial(denialMap, DNSName("b.powerdns.com."), QType::A, false, false);
   BOOST_CHECK_EQUAL(denialState, dState::NODENIAL);
@@ -406,7 +469,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_nxdomain_denial_missing_wildcard)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   /* Add NSEC3 for the closest encloser */
   recordContents.clear();
@@ -419,7 +482,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_nxdomain_denial_missing_wildcard)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   dState denialState = getDenial(denialMap, DNSName("a.powerdns.com."), QType::A, false, false);
   BOOST_CHECK_EQUAL(denialState, dState::NODENIAL);
@@ -448,7 +511,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_expanded_wildcard_proof)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a.example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a.example.org."), QType::NSEC)] = pair;
 
   /* This is an expanded wildcard proof, meaning that it does prove that the exact name
      does not exist so the wildcard can apply */
@@ -479,7 +542,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_wildcard_with_cname)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a.example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a.example.org."), QType::NSEC)] = pair;
 
   /* add a NSEC proving that a wildcard exists, without a CNAME type */
   recordContents.clear();
@@ -492,7 +555,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_wildcard_with_cname)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(DNSName("*.example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("*.example.org."), QType::NSEC)] = pair;
 
   /* A does exist at the wildcard, AAAA does not */
   dState denialState = getDenial(denialMap, DNSName("b.example.org."), QType::A, false, true);
@@ -512,7 +575,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_wildcard_with_cname)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(DNSName("*.example.org."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("*.example.org."), QType::NSEC)] = pair;
 
   /* A and AAAA do not exist but we have a CNAME so at the wildcard */
   denialState = getDenial(denialMap, DNSName("b.example.org."), QType::A, false, true);
@@ -544,7 +607,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_wildcard_with_cname)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   /* Add NSEC3 for the closest encloser */
   recordContents.clear();
@@ -557,7 +620,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_wildcard_with_cname)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   /* add wildcard, without a CNAME type */
   recordContents.clear();
@@ -570,7 +633,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_wildcard_with_cname)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   /* A does exist at the wildcard, AAAA does not */
   dState denialState = getDenial(denialMap, DNSName("b.example.org."), QType::A, false, true);
@@ -590,7 +653,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_wildcard_with_cname)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   /* A and AAAA do not exist but we have a CNAME so at the wildcard */
   denialState = getDenial(denialMap, DNSName("b.example.org."), QType::A, false, true);
@@ -622,7 +685,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_ent_denial)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(DNSName("a.powerdns.com."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName("a.powerdns.com."), QType::NSEC)] = pair;
 
   /* this NSEC is valid to prove a NXQTYPE at c.powerdns.com because it proves that
      it is an ENT */
@@ -648,7 +711,7 @@ BOOST_AUTO_TEST_CASE(test_nsec_ent_denial)
   records.clear();
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(DNSName(").powerdns.com."), QType::NSEC)] = pair;
+  denialMap[std::pair(DNSName(").powerdns.com."), QType::NSEC)] = pair;
 
   denialState = getDenial(denialMap, DNSName("b.powerdns.com."), QType::A, true, false);
   BOOST_CHECK_EQUAL(denialState, dState::NXDOMAIN);
@@ -686,7 +749,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_ancestor_nxqtype_denial)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
   records.clear();
 
   /* RFC 6840 section 4.1 "Clarifications on Nonexistence Proofs":
@@ -716,7 +779,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_ancestor_nxqtype_denial)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   /* add wildcard denial */
   recordContents.clear();
@@ -729,10 +792,14 @@ BOOST_AUTO_TEST_CASE(test_nsec3_ancestor_nxqtype_denial)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   denialState = getDenial(denialMap, DNSName("sub.a."), QType::A, false, true);
   BOOST_CHECK_EQUAL(denialState, dState::NODENIAL);
+
+  /* not even the DS! */
+  denialState = getDenial(denialMap, DNSName("sub.a."), QType::DS, false, true);
+  BOOST_CHECK_EQUAL(denialState, dState::NODENIAL);
 }
 
 BOOST_AUTO_TEST_CASE(test_nsec3_denial_too_many_iterations)
@@ -757,7 +824,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_denial_too_many_iterations)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
   records.clear();
 
   dState denialState = getDenial(denialMap, DNSName("a."), QType::A, false, true);
@@ -798,7 +865,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_insecure_delegation_denial)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
   records.clear();
 
   /* Insecure because the NS is not set, so while it does
@@ -840,7 +907,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_ent_opt_out)
   pair.records = recordContents;
   pair.signatures = signatureContents;
   cspmap_t denialMap;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   /* it can not be used to deny any RRs below that owner name either */
   /* Add NSEC3 for the next closer */
@@ -854,7 +921,7 @@ BOOST_AUTO_TEST_CASE(test_nsec3_ent_opt_out)
 
   pair.records = recordContents;
   pair.signatures = signatureContents;
-  denialMap[std::make_pair(records.at(0).d_name, records.at(0).d_type)] = pair;
+  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
 
   /* Insecure because the opt-out bit is set */
   dState denialState = getDenial(denialMap, DNSName("ent.was.here."), QType::A, false, true);
